package de.laliluna.search;

import java.util.List;

import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.index.Term;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.WildcardQuery;
import org.hibernate.Session;
import org.hibernate.search.FullTextSession;
import org.hibernate.search.Search;
import org.hibernate.search.query.dsl.QueryBuilder;

public class SearchTest {
	public static void main(String[] args) {
		createSomeArticles();
		search();
	}

	private static void search() {
		Session session = SessionFactoryUtil.getFactory().getCurrentSession();
//		create a full text session
		FullTextSession fSession = Search.getFullTextSession(session);
		fSession.beginTransaction();
//		create a parser to parse a full text query
        QueryBuilder qb = fSession.getSearchFactory()
                .buildQueryBuilder().forEntity( Article.class ).get();
        Query query = qb.bool().must(qb.keyword().onField("title").matching("About nonsense").createQuery())
                .must(qb.keyword().onField("content").matching("everywhere").createQuery()).createQuery();


//		execute the query
		List<Article> articles = fSession.createFullTextQuery(query, Article.class).list();
		for (Article article : articles) {
			System.out.println(article);
		}
		System.out.println("Searching for title 'About Hibernate'");
//		search a full term
		articles = fSession.createFullTextQuery(qb.phrase().onField("title").sentence("About Hibernate").createQuery(), Article.class).list();
		for (Article article : articles) {
			System.out.println(article);
		}

		System.out.println("Searching for title starting with 'About'");
//		search a full term
		articles = fSession.createFullTextQuery(qb.keyword().wildcard().onField("title").matching("About*").createQuery(), Article.class).list();
		for (Article article : articles) {
			System.out.println(article);
		}

		fSession.getTransaction().commit();
	}

	private static void createSomeArticles() {
		Session  session = SessionFactoryUtil.getFactory().getCurrentSession();
		session.beginTransaction();
		session.save(new Article("About Hibernate", "This article covers all the quirks of Hibernate"));
		session.save(new Article("About Advanced Hibernate", "Hibernating is a popular sport of students everywhere in the world"));
		session.save(new Article("About Nonsense", "Nonsense is nearly everywhere in our everydays live"));
		
		session.getTransaction().commit();
	}
}
