
package de.laliluna.inheritance.singletable;

import java.util.Iterator;
import java.util.List;

import org.hibernate.LockMode;
import org.hibernate.LockOptions;
import org.hibernate.Session;

import de.laliluna.example.BasicTest;
import de.laliluna.hibernate.InitSessionFactory;

public class TestSingleTable extends BasicTest {

	public static void main(String args[]) {
		Plant plant = create();
		update(plant);
		query();

	}

	private static Plant create() {
		Session session = InitSessionFactory.getInstance().openSession();
		session.beginTransaction();

		Plant plant = new Plant(null, "some type of bush");
		Flower flower = new Flower(null, "rose", "red");
		Tree tree = new Tree(null, "apple tree", true);
		Garden garden = new Garden(null, "my mixed garden");
		garden.getPlants().add(plant);
		// garden.getPlants().add(flower);
		garden.getPlants().add(tree);

		garden.getFlowers().add(flower);

		/*
         * the following will not work garden.getFlowers().add(plant);
         * garden.getFlowers().add(tree);
         */
		session.save(plant);
		session.save(flower);
		session.save(tree);
		session.save(garden);
		session.getTransaction().commit();
		/*
         * normally, configure my Hibernate sessionfactory to autoclose the
         * session after the commit. I check for this in case you did not choose
         * autoclose
         */
		if (session.isOpen())
			session.close();

		session = InitSessionFactory.getInstance().openSession();
		session.beginTransaction();
		garden = (Garden) session.get(Garden.class, garden.getId());
		System.out.println("## Flowers ##");
		for (Iterator iter = garden.getFlowers().iterator(); iter.hasNext();)
			System.out.println(iter.next());
		System.out.println("## Plants ##");
		for (Iterator iter = garden.getPlants().iterator(); iter.hasNext();)
			System.out.println(iter.next());
		session.getTransaction().commit();
		/*
         * normally, I configure my Hibernate sessionfactory to autoclose the
         * session after the commit. I check for this in case you did not choose
         * autoclose
         */
		if (session.isOpen())
			session.close();
		// we return a flower as plant
		return flower;
	}

	private static void update(Plant plant) {
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		// we expect the session to be closed and the plant to be in status
		// detached, so attach it first to move it to status persistent
		session.buildLockRequest(LockOptions.NONE).lock(plant);
		// if we cast to flower we can set the color
		((Flower) plant).setColor("blue");
		session.getTransaction().commit();
	}

	private static void query() {
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		// list plants and all subclasses
		List list = session.createQuery("from Plant").list();
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			Plant element = (Plant) iter.next();
			log.debug("{}", element);
		}
		/*
         * select all flowers with blue color, the following work but I consider
         * this as untidy may be Hibernate will not support this later, so do
         * not use this.
         */
		list = session.createQuery("from Plant p where p.color = 'blue' ")
				.list();
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			Plant element = (Plant) iter.next();
			log.debug("{}", element);
		}
		// the better solution
		list = session.createQuery("from Flower f where f.color = 'blue' ")
				.list();
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			Plant element = (Plant) iter.next();
			log.debug("{}", element);
		}
		// select all gardens with blue colors
		list = session.createQuery(
				"select g from Garden g left join g.flowers f where f.color = 'blue' ").list();
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			Garden element = (Garden) iter.next();
			log.debug("{}", element);
		}

		session.getTransaction().commit();

	}
}