
package de.laliluna.relation.one2one;

import java.util.Iterator;
import java.util.List;
import java.util.Random;

import org.hibernate.LockMode;
import org.hibernate.LockOptions;
import org.hibernate.Session;

import de.laliluna.example.BasicTest;
import de.laliluna.hibernate.InitSessionFactory;


public class TestOne2One extends BasicTest {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Order1 order = createCascade1();
		delete1(order);
		query1();
		clean1();

		Order2 order2 = createCascade2();
		delete2(order2);
		query2();
		clean2();

		test3();
	}

	private static void clean1() {
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		session.createQuery("delete from Order1").executeUpdate();
		session.createQuery("delete from Invoice1").executeUpdate();

		session.getTransaction().commit();

	}

	private static void query1() {
		log.debug("query1");
		// create some orders and invoices
		Random random = new Random();
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		for (int i = 0; i < 20; i++)
		{
			Order1 order = new Order1(null, "" + random.nextInt(30));
			Invoice1 invoice = new Invoice1(null, "" + random.nextInt(30));
			order.setInvoice(invoice);
			session.save(order);
		}
		session.getTransaction().commit();

		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();

		log.debug("########### all orders and initialize the invoice");
		List list = session.createQuery("from Order1 o fetch all properties")
				.list();
		for (Iterator iter = list.iterator(); iter.hasNext();)
		{
			Order1 element = (Order1) iter.next();
			log.debug("{}", element);
			log.debug("{}", element.getInvoice());
		}
		session.getTransaction().commit();

		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();

		log
				.debug("########### all orders but do not initialize the invoice (does not work for this kind of relation)");
		list = session.createQuery("from Order1").list();
		for (Iterator iter = list.iterator(); iter.hasNext();)
		{
			Order1 element = (Order1) iter.next();
			log.debug("{}", element);
			log.debug("{}", element.getInvoice());
		}

		session.getTransaction().commit();
		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();

		log.debug("########### all orders and initialize using fetch ");
		list = session.createQuery("from Order1 o left join fetch o.invoice")
				.list();
		for (Iterator iter = list.iterator(); iter.hasNext();)
		{
			Order1 element = (Order1) iter.next();
			log.debug("{}", element);
			log.debug("{}", element.getInvoice());
		}

		session.getTransaction().commit();
		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();

		log.debug("########### orders where invoice number starts with 2 ");
		list = session.createQuery(
				"from Order1 o where o.invoice.number like '2%' ").list();
		for (Iterator iter = list.iterator(); iter.hasNext();)
		{
			Order1 element = (Order1) iter.next();
			log.debug("{}", element);
			log.debug("{}", element.getInvoice());
		}

		session.getTransaction().commit();

		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();

		log
				.debug("########### select some invoices where order number starts with 1 ");
		list = session.createQuery(
				"select o.invoice from Order1 o where o.number like '1%' ").list();
		for (Iterator iter = list.iterator(); iter.hasNext();)
		{
			Invoice1 element = (Invoice1) iter.next();
			log.debug("{}", element);
		}

		session.getTransaction().commit();

	}

	private static void delete1(Order1 order) {
		log.debug("delete");
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		// order is detachted because the old session is closed.
		session.update(order); // cascade also updates the invoice
		session.delete(order.getInvoice());
		order.setInvoice(null);
		session.getTransaction().commit();
		/*
		 * we do not need to issue a session.save(order) order is in persistent
		 * state, os it will be saved when we commit
		 */

	}

	private static Order1 createCascade1() {
		log.debug("createCascade");
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		Order1 order = new Order1(null, "123");
		Invoice1 invoice = new Invoice1(null, "456");
		order.setInvoice(invoice);
		session.save(order); // cascade will save invoice as well
		session.getTransaction().commit();
		return order;
	}

	private static void clean2() {
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		session.createQuery("delete from Order2").executeUpdate();
		session.createQuery("delete from Invoice2").executeUpdate();
		session.getTransaction().commit();
	}

	private static void query2() {
		log.debug("query2");
		// create some orders and invoices
		Random random = new Random();
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		for (int i = 0; i < 20; i++)
		{
			Order2 order = new Order2(null, "" + random.nextInt(30));
			Invoice2 invoice = new Invoice2(null, "" + random.nextInt(30));
			order.setInvoice(invoice);
			session.save(order);
		}
		session.getTransaction().commit();

		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();

		log.debug("########### orders where invoice number starts with 2 ");
		List list = session.createQuery(
				"from Order2 o where o.invoice.number like '2%' ").list();
		for (Iterator iter = list.iterator(); iter.hasNext();)
		{
			Order2 element = (Order2) iter.next();
			log.debug("{}", element);
			log.debug("{}", element.getInvoice());
		}

		session.getTransaction().commit();

		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();

		log
				.debug("########### select some invoices where order number starts with 1 ");
		list = session.createQuery(
				"select i from Invoice2 i where i.order.number like '1%' ").list();
		for (Iterator iter = list.iterator(); iter.hasNext();)
		{
			Invoice2 element = (Invoice2) iter.next();
			log.debug("{}", element);
			log.debug("{}", element.getOrder());
		}

		session.getTransaction().commit();

	}

	private static void delete2(Order2 order) {
		log.debug("delete");
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		// order is detachted because the old session is closed.
		session.update(order); // cascade also updates the invoice
		session.delete(order.getInvoice());
		order.setInvoice(null);
		session.getTransaction().commit();
		/*
		 * we do not need to issue a session.save(order) order is in persistent
		 * state, os it will be saved when we commit
		 */

	}

	private static Order2 createCascade2() {
		log.debug("createCascade");
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		Order2 order = new Order2(null, "123");
		Invoice2 invoice = new Invoice2(null, "456");
		// bi-directional set on both sides !!!
		order.setInvoice(invoice);
		invoice.setOrder(order);
		session.save(order); // cascade will save invoice as well
		session.getTransaction().commit();
		return order;
	}

	private static void test3() {
		log.debug("Creating order3, invoice3");
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		Order3 order = new Order3(null, "123");
		Invoice3 invoice3 = new Invoice3(null, "456");
		invoice3.setOrder(order);
		session.save(order);
		session.save(invoice3);
		session.getTransaction().commit();
		
		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		session.buildLockRequest(LockOptions.NONE).lock(order);
		session.delete(order);
		session.getTransaction().commit();
	}
}
