
package de.laliluna.example;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.hibernate.Criteria;
import org.hibernate.FetchMode;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projection;
import org.hibernate.criterion.ProjectionList;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Property;
import org.hibernate.criterion.Restrictions;
import org.hibernate.criterion.Subqueries;
import org.hibernate.transform.AliasToBeanResultTransformer;
import org.hibernate.transform.DistinctRootEntityResultTransformer;

import de.laliluna.example13.Cupboard;
import de.laliluna.example13.PoolOwner;
import de.laliluna.example13.SwimmingPool;
import de.laliluna.example13.Room;
import de.laliluna.hibernate.InitSessionFactory;
import de.laliluna.relation.one2many.JavaClub2;
import de.laliluna.relation.one2many.JavaClub3;
import de.laliluna.relation.one2many.JavaClubMember3;
import de.laliluna.relation.overview.Developer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class TestQuery {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// selects();
		// whereClauses();
		// selectCriterias();
		// criteriaQueries();
		// nativeSql();
		// subQueriesHQL();
		subQueriesCriteria();
	}
    static Logger log = LoggerFactory.getLogger(TestQuery.class);

	private static void subQueriesCriteria() {
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		/* create some sample data if the table is empty */
		List pools = session.createCriteria(SwimmingPool.class).list();
		if (pools.size() == 0)
		{
			SwimmingPool p[] = new SwimmingPool[7];
			p[0] = new SwimmingPool("Frankfurt", 20);
			p[1] = new SwimmingPool("Frankfurt", 40);
			p[2] = new SwimmingPool("Dresden", 18);
			p[3] = new SwimmingPool("Leipzig", 70);
			p[4] = new SwimmingPool("Stuttgart", 30);
			p[5] = new SwimmingPool("Lemgo", 77);
			p[6] = new SwimmingPool("Flensburg", 40);
			PoolOwner po[] = new PoolOwner[5];
			po[0] = new PoolOwner("Sebastian");
			po[0].getSwimmingPools().add(p[0]);
			po[0].getSwimmingPools().add(p[1]);
			po[0].getSwimmingPools().add(p[6]);
			po[1] = new PoolOwner("Jim");
			po[1].getSwimmingPools().add(p[5]);
			po[2] = new PoolOwner("Bim");
			po[2].getSwimmingPools().add(p[2]);
			po[3] = new PoolOwner("Bum");
			po[3].getSwimmingPools().add(p[3]);
			po[4] = new PoolOwner("Bam");
			po[4].getSwimmingPools().add(p[4]);

			for (int i = 0; i < p.length; i++)
				session.save(p[i]);
			for (int i = 0; i < po.length; i++)
				session.save(po[i]);

		}
		session.getTransaction().commit();
		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();

		log
				.debug("Select all pools not beeing in Frankfurt which size is  bigger than all pools in Frankfurt");
		DetachedCriteria detachedCriteria = DetachedCriteria.forClass(
				SwimmingPool.class).setProjection(Projections.property("size")).add(
				Restrictions.eq("city", "Frankfurt"));
		pools = session.createCriteria(SwimmingPool.class).add(
				Restrictions.ne("city", "Frankfurt")).add(
				Property.forName("size").gtAll(detachedCriteria)).list();
		for (Iterator iter = pools.iterator(); iter.hasNext();)
		{
			SwimmingPool pool = (SwimmingPool) iter.next();
			log.debug("{}", pool);
		}

		log
				.debug("Select all pools not beeing in Frankfurt which size is at least bigger than one pool in Frankfurt");
		detachedCriteria = DetachedCriteria.forClass(SwimmingPool.class)
				.setProjection(Projections.property("size")).add(
						Restrictions.eq("city", "Frankfurt"));
		pools = session.createCriteria(SwimmingPool.class).add(
				Restrictions.ne("city", "Frankfurt")).add(
				Property.forName("size").gtSome(detachedCriteria)).list();
		for (Iterator iter = pools.iterator(); iter.hasNext();)
		{
			SwimmingPool pool = (SwimmingPool) iter.next();
			log.debug("{}", pool);
		}

		log.debug("Select all pools having the same size than a pool in Frankfurt");
		detachedCriteria = DetachedCriteria.forClass(SwimmingPool.class)
				.setProjection(Projections.property("size")).add(
						Restrictions.eq("city", "Frankfurt"));
		pools = session.createCriteria(SwimmingPool.class).add(
				Restrictions.ne("city", "Frankfurt")).add(
				Property.forName("size").in(detachedCriteria)).list();
		for (Iterator iter = pools.iterator(); iter.hasNext();)
		{
			SwimmingPool pool = (SwimmingPool) iter.next();
			log.debug("{}", pool);
		}

		log
				.debug("Select all PoolOwner having a pool in Frankfurt which size is greater than 20");
		detachedCriteria = DetachedCriteria.forClass(SwimmingPool.class).add(
				Restrictions.gt("size", 20)).add(Restrictions.ne("city", "Frankfurt")).setProjection(Projections.property("id"));
		pools = session.createCriteria(PoolOwner.class).createAlias("swimmingPools", "s").add(
				Subqueries.propertyIn("s.id", detachedCriteria)).list();
		for (Iterator iter = pools.iterator(); iter.hasNext();)
		{
			PoolOwner pool = (PoolOwner) iter.next();
			log.debug("{}", pool);
		}
		session.getTransaction().commit();

	}

	private static void subQueriesHQL() {

		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		/* create some sample data if the table is empty */
		List pools = session.createCriteria(SwimmingPool.class).list();
		if (pools.size() == 0)
		{
			SwimmingPool p[] = new SwimmingPool[7];
			p[0] = new SwimmingPool("Frankfurt", 20);
			p[1] = new SwimmingPool("Frankfurt", 40);
			p[2] = new SwimmingPool("Dresden", 18);
			p[3] = new SwimmingPool("Leipzig", 70);
			p[4] = new SwimmingPool("Stuttgart", 30);
			p[5] = new SwimmingPool("Lemgo", 77);
			p[6] = new SwimmingPool("Flensburg", 40);
			PoolOwner po[] = new PoolOwner[5];
			po[0] = new PoolOwner("Sebastian");
			po[0].getSwimmingPools().add(p[0]);
			po[0].getSwimmingPools().add(p[1]);
			po[0].getSwimmingPools().add(p[6]);
			po[1] = new PoolOwner("Jim");
			po[1].getSwimmingPools().add(p[5]);
			po[2] = new PoolOwner("Bim");
			po[2].getSwimmingPools().add(p[2]);
			po[3] = new PoolOwner("Bum");
			po[3].getSwimmingPools().add(p[3]);
			po[4] = new PoolOwner("Bam");
			po[4].getSwimmingPools().add(p[4]);

			for (int i = 0; i < p.length; i++)
				session.save(p[i]);
			for (int i = 0; i < po.length; i++)
				session.save(po[i]);

		}
		session.getTransaction().commit();
		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		log
				.debug("Select all pools not beeing in Frankfurt which size is bigger than the biggest pools in Frankfurt");
		pools = session
				.createQuery(
						"from SwimmingPool p where p.city <> 'Frankfurt' and p.size > all (select p2.size from SwimmingPool p2 where p2.city like 'Frankfurt')")
				.list();
		for (Iterator iter = pools.iterator(); iter.hasNext();)
		{
			SwimmingPool pool = (SwimmingPool) iter.next();
			log.debug("{}", pool);
		}
		log
				.debug("Select all pools not beeing in Frankfurt which size is at least bigger than one pool in Frankfurt");
		pools = session
				.createQuery(
						"from SwimmingPool p where p.city <> 'Frankfurt' and  p.size > any (select p2.size from SwimmingPool p2 where p2.city like 'Frankfurt')")
				.list();
		for (Iterator iter = pools.iterator(); iter.hasNext();)
		{
			SwimmingPool pool = (SwimmingPool) iter.next();
			log.debug("{}", pool);
		}
		log.debug("Select all pools having the same size as a pool in Frankfurt");
		pools = session
				.createQuery(
						"select p from SwimmingPool p where p.city <> 'Frankfurt' and p.size in (select p2.size from SwimmingPool p2 where p2.city like 'Frankfurt')")
				.list();
		for (Iterator iter = pools.iterator(); iter.hasNext();)
		{
			SwimmingPool pool = (SwimmingPool) iter.next();
			log.debug("{}", pool);
		}
		log.debug("Select all PoolOwner having a swimming pool");
		pools = session.createQuery(
				"select o from PoolOwner o where exists elements(o.swimmingPools)")
				.list();
		for (Iterator iter = pools.iterator(); iter.hasNext();)
		{
			PoolOwner poolOwner = (PoolOwner) iter.next();
			log.debug("{}", poolOwner);
		}
		log.debug("Select all PoolOwner having a swimming pool greater than 20");
		pools = session
				.createQuery(
						"select o from PoolOwner o where elements(o.swimmingPools) in (select p from SwimmingPool p where p.size > 20)")
				.list();
		for (Iterator iter = pools.iterator(); iter.hasNext();)
		{
			PoolOwner poolOwner = (PoolOwner) iter.next();
			log.debug("{}", poolOwner);
		}

		session.getTransaction().commit();

	}

	private static void nativeSql() {
		/*
		 * I expect that you run the test code for the example3 mappings in order to
		 * have some test data generated. If not, please do so before running these
		 * examples.
		 * 
		 */
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();

		/* select and join with standard sql */
		List results = session
				.createSQLQuery(
						"select c.id, c.name from  tjavaclub c left join tjavaclubmember m on c.id=m.club_id"
								+ " where m.name='Peter' ").list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			Object[] objects = (Object[]) iter.next();
			Integer id = (Integer) objects[0];
			String name = (String) objects[1];
			log.debug("Club Id: " + id + " Name: " + name);
		}

		/*
		 * the following will not work, you cannot update with native sql, Hibernate
		 * does not want it. Get the JDBC connection from the
		 * HibernateSessionFactory instead and use it for normal SQL queries.
		 * 
		 * session.createSQLQuery( "update tjavaclub set name='new name' where id
		 * =5") .executeUpdate();
		 */
		session.getTransaction().commit();

	}

	private static void criteriaQueries() {
		/*
		 * I expect that you run the test code for the example3 mappings in order to
		 * have some test data generated. If not, please do so before running these
		 * examples.
		 * 
		 */
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();

		/* one single result */
		JavaClub3 aClub = (JavaClub3) session.createCriteria(JavaClub3.class).add(
				Restrictions.eq("id", 5)).uniqueResult();
		log.debug("one single club: " + aClub);

		/*
		 * where condition in a relation, we need a second criteria and do not have
		 * to join
		 */
		List results = session.createCriteria(JavaClub3.class).createCriteria(
				"members").add(Restrictions.eq("name", "Peter")).list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 club3 = (JavaClub3) iter.next();
			log.debug("Club with member named Peter: " + club3);
		}

		/*
		 * where condition with grouping which is equivalent to and/or with normal
		 * HQL se select all clubs with id > than 5 or with a name starting with
		 * Java where there is a member named Peter
		 */

		// TODO when you use Java 1.4 or older you must replace the number 5
		results = session.createCriteria(JavaClub3.class).add(
				Restrictions.disjunction().add(Restrictions.eq("name", "Java%")).add(
						Restrictions.gt("id", 5))).createCriteria("members").add(
				Restrictions.eq("name", "Peter")).list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 club3 = (JavaClub3) iter.next();
			log.debug("Club with member named Peter: " + club3);
		}

		/* walking through a 1:1 relation */
		results = session.createCriteria(Cupboard.class).createCriteria("lock")
				.createCriteria("key").add(Restrictions.eq("name", "old key")).list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			Cupboard cupboard = (Cupboard) iter.next();
			log.debug("{}", cupboard);
		}

		/* walking through relations with 1:n */
		results = session.createCriteria(Room.class).createCriteria("cupboards")
				.createCriteria("lock").createCriteria("key").add(
						Restrictions.eq("name", "old key")).list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			Room room = (Room) iter.next();
			log.debug("{}", room);
		}

		session.getTransaction().commit();

	}

	private static void whereClauses() {
		/*
		 * I expect that you run the test code for the example3 mappings in order to
		 * have some test data generated. If not, please do so before running these
		 * examples.
		 * 
		 */
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();

		/* one single result */
		JavaClub3 aClub = (JavaClub3) session.createQuery(
				"from JavaClub3 where id=5").uniqueResult();
		log.debug("one single club: " + aClub);

		/* same with use of alias */
		aClub = (JavaClub3) session.createQuery("from JavaClub3 c where c.id=5")
				.uniqueResult();
		log.debug("one single club: " + aClub);

		/*
		 * where condition in a relation, we have to use join if we have an 1:n
		 * relation
		 */
		List results = session
				.createQuery(
						"select c from JavaClub3 c left join c.members m where m.name='Peter' ")
				.list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 club3 = (JavaClub3) iter.next();
			log.debug("Club with member named Peter: " + club3);
		}

		/*
		 * where condition with parameters we need to join if we have a 1:n relation
		 */

		Query query = session
				.createQuery("select c from JavaClub3 c left join c.members m "
						+ "where c.id > ? and m.name like ?");
		// TODO when you use Java 1.4 or older you must replace the number 5
		// with new Integer(5)
		results = query.setInteger(0, 5).setString(1, "Peter%").list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 javaClub3 = (JavaClub3) iter.next();
			log.debug("Some clubs: " + javaClub3);
		}

		/* where condition comparing an object */

		JavaClub3 club3 = (JavaClub3) session.createQuery("from JavaClub3")
				.setMaxResults(1).uniqueResult();

		results = session.createQuery("from JavaClubMember3 m where m.club = ?")
				.setEntity(0, club3).list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClubMember3 member = (JavaClubMember3) iter.next();
			log.debug("Member in the club: " + club3 + " is: " + member);
		}

		/*
		 * where condition comparing an object on the multipe side -> we need in
		 * elements
		 */

		JavaClubMember3 member3 = (JavaClubMember3) session.createQuery(
				"from JavaClubMember3").setMaxResults(1).uniqueResult();

		results = session.createQuery(
				"from JavaClub3 c where ? in elements (c.members)").setEntity(0,
				member3).list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 javaClub3 = (JavaClub3) iter.next();
			log.debug("Member: " + member3 + " is in the club: " + javaClub3);
		}

		session.getTransaction().commit();

	}

	private static void selects() {
		/*
		 * I expect that you run the test code for the example3 mappings in order to
		 * have some test data generated. If not, please do so before running these
		 * examples.
		 * 
		 */
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		List results = session.createQuery("from JavaClub3 ").list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 club3 = (JavaClub3) iter.next();
			log.debug("{}", club3);
		}

		// select an object where you expect a unique result
		JavaClub3 aClub = (JavaClub3) session.createQuery(
				"from JavaClub3 where id=5").uniqueResult();
		log.debug("one single club: " + aClub);

		/*
		 * select with join delivers an object array
		 * 
		 */
		log.debug("select with join delivers an object array");
		results = session.createQuery("from JavaClub3 c left join c.members")
				.list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			Object object[] = (Object[]) iter.next();

			log.debug("Club:  {} Member {}",  object[0], object[1]);
		}

		/*
		 * select JavaClub3 and join members but do not create an object array will
		 * result in multiple JavaClub3 entries if there are more than 1 member. Do
		 * not forget it is a join.
		 * 
		 */
		results = session
				.createQuery(
						"select c from JavaClub3 c left join fetch c.members m where m.name='Peter'")
				.list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 javaClub3 = (JavaClub3) iter.next();
			log.debug("Club: {} ", javaClub3);
		}
		/*
		 * select JavaClub3 and join members but only get one instance of JavaClub3
		 * even after joining. This is useful for initializing.
		 * 
		 */
		/* approach 1 use Hashset, you will loose the sort order */
		log.debug("select with join returning distinct results");
		Set setResults = new HashSet(session.createQuery(
				"select c from JavaClub3 c left join c.members  order by c.name desc")
				.list());
		for (Iterator iter = setResults.iterator(); iter.hasNext();)
		{
			JavaClub3 club3 = (JavaClub3) iter.next();
			log.debug("Club:  " + club3);
		}

		log.debug("the next approach keeps the ordering of your query as well ");
		results = session
						.createQuery(
								"select c from JavaClub3 c left join c.members  order by c.name desc")
                .setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY)
						.list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 club3 = (JavaClub3) iter.next();
			log.debug("Club:  " + club3);
		}

		/* selecting properties */
		results = session.createQuery("select c.id from JavaClub3 c").list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			Integer clubId = (Integer) iter.next();
			log.debug("Club Id:  " + clubId);
		}

		log.debug("Select scalar values (=properties) from a class");
		results = session.createQuery("select c.id, c.name from JavaClub3 c")
				.list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			Object object[] = (Object[]) iter.next();

			log.debug("Club Id:  " + object[0] + " name: " + object[1]);
		}

		/*
		 * Selecting objects and scalar values
		 */
		log.debug("Select the object and scalar values at the same time.");
		results = session.createQuery(
				"select c, m.name from JavaClub3 c left join c.members m").list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			Object object[] = (Object[]) iter.next();
			JavaClub3 club3 = (JavaClub3) object[0];
			String name = (String) object[1];
			log.debug("Club:  " + club3 + " Member name: " + name);
		}

		/* creating an object with selective properties */
		log.debug("creating an object with selective properties");
		results = session.createQuery(
				"select new JavaClub3(c.name) from JavaClub3 c").list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 element = (JavaClub3) iter.next();
			log.debug("{}", element.getName());
		}

		/* walking through a 1:1 relation */
		results = session.createQuery("from Cupboard c where c.lock.key.name = ?")
				.setString(0, "old key").list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			Cupboard cupboard = (Cupboard) iter.next();
			log.debug("{}", cupboard);
		}

		/* walking through relations with 1:n */
		results = session
				.createQuery(
						"select r from Room r left join r.cupboards c where c.lock.key.name = ?")
				.setString(0, "old key").list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			Room room = (Room) iter.next();
			log.debug("{}", room);
		}

		session.getTransaction().commit();

	}

	public static void selectCriterias() {
		/*
		 * I expect that you run the test code for the example3 mappings in order to
		 * have some test data generated. If not, please do so before running these
		 * examples.
		 * 
		 */
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		List results = session.createCriteria(JavaClub3.class).list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 club3 = (JavaClub3) iter.next();
			log.debug("{}", club3);
		}

		// select an object where you expect a unique result
		JavaClub3 aClub = (JavaClub3) session.createCriteria(JavaClub3.class).add(
				Restrictions.eq("id", 5)).uniqueResult();
		log.debug("one single club: " + aClub);

		/*
		 * if we fetch related data, it will be fetched by join or subselect The
		 * result will contain multiple entries for a JavaClub if it has more than
		 * one member. Think of the structure of the SQL join to understand this.
		 */
		log.debug("Selecting with fetching does not result in an array of objects");
		results = session.createCriteria(JavaClub3.class).setFetchMode("members",
				FetchMode.JOIN).list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 javaClub3 = (JavaClub3) iter.next();
			log.debug("Club:  " + javaClub3);
		}

		log.debug("Select with fetching with a result transformer");
		results = session.createCriteria(JavaClub3.class).addOrder(
				Order.desc("name")).setFetchMode("members", FetchMode.JOIN)
				.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY).list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 javaClub3 = (JavaClub3) iter.next();
			log.debug("Club:  " + javaClub3);
		}

		/*
		 * select Javaclub and join Member does not create an object array as in
		 * HQL. It will result in multiple Member entries if there are more than 1
		 * computer. Do not forget it is a join.
		 * 
		 */
		log.debug("Selecting an object with a where condition on a related object");
		results = session.createCriteria(JavaClub3.class).createCriteria("members")
				.add(Restrictions.eq("name", "Peter")).list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 javaClub3 = (JavaClub3) iter.next();
			log.debug("Club:  " + javaClub3);
		}

		log.debug("Selecting a single property");
		results = session.createCriteria(JavaClub3.class).setProjection(
				Projections.property("id")).list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			Integer clubId = (Integer) iter.next();

			log.debug("Club Id:  " + clubId);
		}

		log.debug("Selecting multipe single properties");
		results = session.createCriteria(JavaClub3.class).setProjection(
				Projections.projectionList().add(Projections.property("id")).add(
						Projections.property("name"))).list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			Object object[] = (Object[]) iter.next();

			log.debug("Club Id:  " + object[0] + " name: " + object[1]);
		}

		/*
		 * Selecting objects and scalar values at the same time does not work
		 */

		/* creating an object with selective properties */
		results = session.createCriteria(JavaClub3.class).setProjection(
				Projections.property("name")).setResultTransformer(
				new AliasToBeanResultTransformer(JavaClub3.class)).list();
		for (Iterator iter = results.iterator(); iter.hasNext();)
		{
			JavaClub3 element = (JavaClub3) iter.next();
			log.debug("{}", element.getName());
		}
	}
}
