package com.example.newapp.pages;

import com.example.newapp.domain.Book;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.hibernate.annotations.CommitAfter;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.hibernate.Session;


public class EditBook {

    /*
    @Property makes the book available to the EditBook.tml page.
     */
    @Property
    private Book book;

    @Inject
    private Session session;

    /*
    Inject a page. It is used later for navigation purpose.
     */
    @InjectPage
    private Index index;

    /**
     * Is called before the page is rendered. A value encoder provided by the
     * Tapestry Hibernate module, knows how to convert the id at the end of the URL
     * localhost:8080/editbook/2 into an instance of {@link Book}
     * @param book the book to edit
     */
    void onActivate(Book book){
        this.book = book;
    }

    /**
     * On redirecting to the same page for example, if validation fails, it
     * is required to make Tapestry aware of the page context object.
     *
     * Otherwise you will loose the information which book was edited.
     * @return the edited book
     */
    Book onPassivate(){
        return book;
    }

    /**
     * Commit the transaction using {@code session.getTransaction().commit()} right after the
     * method was executed.
     */
    @CommitAfter
    Object onSuccess() {
        session.saveOrUpdate(book);
        /*
        Return the injected page to navigate to the page.
         */
        return index;
    }
}
