package de.laliluna.cachetest;

import java.util.Random;


import org.hibernate.Session;
import org.hibernate.Transaction;

import de.laliluna.hibernate.CacheSzenario;
import de.laliluna.hibernate.InitSessionFactory;

public class TestMain {

	private static org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(TestMain.class);

	/**
     * @param args
     */
	public static void main(String[] args) {
		int maxThreads = 1;
		// create test data
		 //setupTest();
		// start a random access test
		startRandomGetThreads(maxThreads, CacheSzenario.READWRITECACHE);
		//startRandomNaturalIdThreads(maxThreads, CacheSzenario.QUERYCACHE, true);
		/**
         * test the query cache with one of the following appraoches
         */
		// testQueryCache with enabled Cache
		//startQueryThreads(maxThreads, CacheSzenario.QUERYCACHE, true);
		// testQueryCache with enabled Cache for swarmcache
		// startQueryThreads(maxThreads, CacheSzenario.NONRESTRICTEDQUERYDCACHE,
		// true);
		// testQueryCache with disabled Cache
		// startQueryThreads(maxThreads, CacheSzenario.QUERYCACHE, false);
	}

	private static void startRandomNaturalIdThreads(int maxThreads,
			CacheSzenario cacheSzenario, boolean cacheQueries) {
		// init
		InitSessionFactory.initSessionFactory(cacheSzenario);
		InitSessionFactory.getInstance();
		// start threads
		RandomNaturalIdThread myThreads[] = new RandomNaturalIdThread[maxThreads];
		for (int i = 0; i < myThreads.length; i++) {
			myThreads[i] = new RandomNaturalIdThread(cacheQueries);
			myThreads[i].start();
		}

	}

	private static void setupTest() {
		int maxEntries = 1000;

		InitSessionFactory.initSessionFactory(CacheSzenario.NOCACHE);
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		Random random = new Random();
		Transaction tx = session.beginTransaction();
		if (session.createQuery("from Computer").list().size() > 0)
			log.debug("will delete Computer: ");
		session.createQuery("delete from Computer").executeUpdate();
		if (session.createQuery("from Developer").list().size() > 0)
			log.debug("will delete Developer: ");
		session.createQuery("delete from Developer").executeUpdate();
		tx.commit();

		log.debug("Start creating test data");
		int computerId = 1;

		session = InitSessionFactory.getInstance().getCurrentSession();
		tx = session.beginTransaction();
		for (int i = 0; i < maxEntries; i++) {
			Developer d = new Developer();
			d.setId(i + 1);
			d.setName("Peter");
			d.setRandom(random.nextInt(100));
			int maxComputer = 2 + random.nextInt(5);
			Computer computers[] = new Computer[maxComputer];
			for (int i2 = 0; i2 < maxComputer; i2++) {
				computers[i2] = new Computer();
				computers[i2].setId(computerId++);
				computers[i2].setName("" + random.nextInt(100));
				// hopefully this produces unique numbers ;-)
				computers[i2].setAdministrationNumber("" + random.nextLong());
				d.getComputers().add(computers[i2]);
				computers[i2].setDeveloper(d);
			}
			
			session.save(d);
			if (i % 100 == 0) {
				session.flush();
				session.clear();
			}
		}
		tx.commit();
		log.debug("Finished creating test data");
	}

	private static void startRandomGetThreads(int maxThreads, CacheSzenario cacheSzenario) {

		// init
		InitSessionFactory.initSessionFactory(cacheSzenario);
		InitSessionFactory.getInstance();
		// start threads
		RandomGetThread myThreads[] = new RandomGetThread[maxThreads];
		for (int i = 0; i < myThreads.length; i++) {
			myThreads[i] = new RandomGetThread();
			myThreads[i].start();
		}

	}

	private static void startQueryThreads(int maxThreads, CacheSzenario cacheSzenario,
			boolean cacheQueries) {

		// init
		InitSessionFactory.initSessionFactory(cacheSzenario);
		InitSessionFactory.getInstance();
		// start threads
		RandomQueryThread myThreads[] = new RandomQueryThread[maxThreads];
		for (int i = 0; i < myThreads.length; i++) {
			myThreads[i] = new RandomQueryThread(cacheQueries);
			myThreads[i].start();
		}

	}

}
